//cl tfp2ini.c

#include <windows.h>
#include <stdio.h>


typedef struct {
    ULONG   Address;                    //use 0 to scan the whole binary
    ULONG   CodeLength;                 //in dword instructions
    ULONG   OriginalCode[0x200];
    ULONG   OriginalMask[0x200];        //not used if Address<>0
    ULONG   PatchedCode[0x200];
    ULONG   PatchMask[0x200];
}type_PatchSegment;

typedef struct {
    ULONG   DestinationGP;              //unaligned      littleendian           //use 0 for all firmware levels
    ULONG   NrOfSegments;               //unaligned      littleendian
    char    Comment[0x50];
    type_PatchSegment   PatchSegment;
}type_PatchDetails;

typedef struct {
    ULONG   Magic;
    ULONG   FileVersion;
    char    Author[0x50];
    char    Name[0x50];
    char    email[0x50];
    char    EnglishDescr[0x100];
    char    GermanDescr[0x100];
    char    Version;
    ULONG   NrOfPatches;                //unaligned      bigendian  ???
    type_PatchDetails   PatchDetails;
}type_PatchModule;


type_PatchSegment PatchSegment;
type_PatchDetails PatchDetails;
type_PatchModule PatchModule;


int main(int argc, char **argv)
{
    FILE *Ip;
    char ifilename[0x200] = "test.tfp";
    FILE *Op;
    char ofilename[0x200] = "test.ini";

    int rlen;
    int len;
    int i,j,k;
    ULONG dummy;
    int NrOfPatches;
    int NrOfSegments;


    j=0;
    for(i=1; i<argc; i++) {
        if ((argv[i][0] == '-') || (argv[i][0] == '/')) {
            switch (argv[i][1]) {
                case 'h' :
                case '?' :
                default  :
                    printf("\ntfp2ini v0.01");
                    printf("\n=============");
                    printf("\n");
                    printf("\nusage: tfp2ini <input file> <output file>");
                    return 1;
            }
        }
        else {
            if (j == 0) {
                sscanf(argv[i],"%[^]]", ifilename);
                j++;
            }
            else if (j == 1) {
                sscanf(argv[i],"%[^]]", ofilename);
                j++;
            }
        }
    }
    printf("\nConverting tfp to ini file");
    printf("\nReading %s", ifilename);
    printf("\nWriting %s", ofilename);

    Ip = fopen(ifilename, "rb");
    if (!Ip) {
        printf("\nError: %s: Canot open file: %s\n", ifilename);
        return -1;
    }
    Op = fopen(ofilename, "wb");
    if (!Op) {
        printf("\nError: %s: Canot open file: %s\n", ofilename);
        fclose(Ip);
        return -1;
    }

    rlen = fread(&PatchModule.Magic, 1, sizeof(ULONG), Ip);
    rlen = fread(&PatchModule.FileVersion, 1, sizeof(ULONG), Ip);
    rlen = fread(&PatchModule.Author, 1, 0x50, Ip);
    rlen = fread(&PatchModule.Name, 1, 0x50, Ip);
    rlen = fread(&PatchModule.email, 1, 0x50, Ip);
    rlen = fread(&PatchModule.EnglishDescr, 1, 0x100, Ip);
    rlen = fread(&PatchModule.GermanDescr, 1, 0x100, Ip);
    rlen = fread(&PatchModule.Version, 1, sizeof(char), Ip);
    rlen = fread(&dummy, 3, sizeof(char), Ip);                       //!!! NrOfPatches = big endian...
    rlen = fread(&PatchModule.NrOfPatches, 1, sizeof(char), Ip);
    NrOfPatches = PatchModule.NrOfPatches;

    if (PatchModule.Magic != 0x1a504654) {
        printf("\nError: %s: Invalid file format, %08x - not TFP\n", ifilename, PatchModule.Magic);
        fclose(Op);
        fclose(Ip);
        return -1;
    }
//    if (PatchModule.FileVersion != 2) {
//        printf("\nError: Invalid version - %d\n", PatchModule.FileVersion);
//        fclose(Op);
//        fclose(Ip);
//        return -1;
//    }

    fprintf(Op, "[Generic]\n");
    fprintf(Op, "Author=%s\n", PatchModule.Author);
    fprintf(Op, "email=%s\n", PatchModule.email);
    fprintf(Op, "Name=%s\n", PatchModule.Name);
    fprintf(Op, "GermanDescr=%s\n", PatchModule.GermanDescr);
    fprintf(Op, "EnglishDesc=%s\n", PatchModule.EnglishDescr);
    fprintf(Op, "Version=%d\n", PatchModule.Version);
    fprintf(Op, "NrOfPatches=%x\n", PatchModule.NrOfPatches);
    for(k=0; k<NrOfPatches; k++) {
        rlen = fread(&PatchDetails.DestinationGP, 1, sizeof(ULONG), Ip);
        rlen = fread(&PatchDetails.NrOfSegments, 1, sizeof(ULONG), Ip);
        NrOfSegments = PatchDetails.NrOfSegments;
        rlen = fread(&PatchDetails.Comment, 1, 0x50, Ip);

        fprintf(Op, "\n");
        fprintf(Op, "[Patch%02d]\n", k+1);
        fprintf(Op, "Comment=%s\n", PatchDetails.Comment);
        fprintf(Op, "DestinationGP=%08x\n", PatchDetails.DestinationGP);
        fprintf(Op, "NrOfSegments=%d\n", PatchDetails.NrOfSegments);

        for(j=0; j<NrOfSegments; j++) {
            rlen = fread(&PatchSegment.Address, 1, sizeof(ULONG), Ip);
            rlen = fread(&PatchSegment.CodeLength, 1, sizeof(ULONG), Ip);
            len = PatchSegment.CodeLength;
            rlen = fread(&PatchSegment.OriginalCode, len, 4, Ip);
            rlen = fread(&PatchSegment.OriginalMask, len, 4, Ip);
            rlen = fread(&PatchSegment.PatchedCode, len, 4, Ip);
            rlen = fread(&PatchSegment.PatchMask, len, 4, Ip);

            fprintf(Op, "\n");
            fprintf(Op, "[Patch%02d_Segment%02d]\n", k+1, j+1);
            fprintf(Op, "Address=%x\n", PatchSegment.Address);
            fprintf(Op, "CodeLength=%d\n", PatchSegment.CodeLength);
            fprintf(Op, "OriginalCode=");
            for (i=0; i<len; i++) fprintf(Op, "%08x ", PatchSegment.OriginalCode[i]);
            fprintf(Op, "\n");
            fprintf(Op, "OriginalMask=");
            for (i=0; i<len; i++) fprintf(Op, "%08x ", PatchSegment.OriginalMask[i]);
            fprintf(Op, "\n");
            fprintf(Op, "PatchedCode=");
            for (i=0; i<len; i++) fprintf(Op, "%08x ", PatchSegment.PatchedCode[i]);
            fprintf(Op, "\n");
            fprintf(Op, "PatchMask=  ");
            for (i=0; i<len; i++) fprintf(Op, "%08x ", PatchSegment.PatchMask[i]);
            fprintf(Op, "\n");
        }
    }

    fclose(Op);
    fclose(Ip);
    return 0;
}
